#!/usr/bin/perl -w -I/usr/sausalito/perl

######################
#  Handler for Arkeia backup server settings
#  ebraswell
#  Copyright 2000, Cobalt Networks.  All rights reserved.

use strict;

use CCE;
use Sauce::Util;
# use Sauce::Validators;

#declares that should probably go elsewhere:
my $linkFile        = "/etc/rc.d/rc3.d/S99arkeia";
my $scriptFile       = "/etc/rc.d/init.d/arkeia";
my $confFile	     = "/usr/knox/nlp/admin.cfg";
my $netConfFile	= '/usr/knox/nlp/nlp.cfg';

my $confBlockStart = "# Arkeia Config Start"; # Comment block for config files
my $confBlockStop = "# Arkeia Config Stop";

my $cce = new CCE ( Namespace => "Arkeia" );

$cce->connectfd();

# Need to test if event_object successful... what do we look at?
my $arkeia_obj = $cce->event_object();
my $arkeiaHost = $arkeia_obj->{arkeiaHost};

my $enabled = $arkeia_obj->{enable};

if ($enabled && !$arkeiaHost) {
    $cce->warn("[[knox-arkeia.enabledButNoData]]");
    $cce->bye("FAIL");
    die;
}

if ($enabled && !checkDNSLookup($arkeiaHost)) {
    $cce->warn("[[knox-arkeia.arkeiaHostField_LookupFailed]]");
    $cce->bye("FAIL");
    die;
}
my $hostname = `hostname`;
chomp($hostname);

if ($enabled && !checkDNSLookup($hostname)) {
    $cce->warn("[[knox-arkeia.arkeiaLocalHost_LookupFailed]]");
    $cce->bye("FAIL");
    die;
}


# update the config file

# Sauce::Util::replaceblock($confFile, $confBlockStart , $arkeiaHost, $confBlockStop);

# Use our own routine to get rid of existing config
Sauce::Util::editfile( $confFile, \&set_arkeia_addr, $arkeiaHost);

# update the port used if necessary
my $new = $cce->event_new();

if ($new->{port}) {
	Sauce::Util::editfile( $netConfFile, \&set_port, $new->{port} );
}

# Start service
# NOTE: Arkeia is in multiple runlevels. This will only get rc3... how 'bout chkconfig?

if ($enabled) {
    		if (! -e $linkFile) { #Link doesn't exist, start service
	        	my $pid;
		        if (!defined($pid = fork)) {
		            $cce->warn("[[knox-arkeia.cannotFork]] $!");
			    $cce->bye("FAIL");
			    die;
		        } elsif ($pid) {
		            # i'm the parent
		        } else {
		            # otherwise I'm the child
		            symlink($scriptFile ,$linkFile) || (
			      $cce->warn("[[knox-arkeia.cannotCreateSymlink]] $linkFile: $!") && $cce->bye("FAIL") && die);
			    system ("$scriptFile start > /dev/null") && (
			      $cce->warn("[[knox-arkeia.cannotStartArkeiaServer]]: $!") && $cce->bye("FAIL") && die);
		            exit;
		        }
		    } else {
			# restart service to reflect changes
			system("$scriptFile stop > /dev/null");
			system("$scriptFile start > /dev/null");
		     }
		
	
	} elsif (-e $linkFile) {	#NO SERVICE, Link exists: kill service and remove link
		#kill service
		system ("$scriptFile stop > /dev/null") && ( 
		$cce->warn("[[knox-arkeia.cannotStopArkeiaServer]]: $!") && $cce->bye("FAIL"));
		unlink($linkFile) || (
		$cce->warn("[[knox-arkeia.cannotBreakLink]] $linkFile: $!") && $cce->bye("FAIL"));
	} else {	#Nothing Changed...
	
	}

$cce->bye("SUCCESS");
# Set new configuration

sub set_arkeia_addr {	

	my ($fin, $fout, $hostaddr) = @_;
	#ignore $fin, old config is backed up and we start frisch

	print $fout " \n$hostaddr \n ";
}

# edit function to parse nlp.cfg and update the port number
sub set_port
{
	my ($in, $out, $port) = @_;

	my $done = 0;

	while (<$in>) {
		if (/PORT_NUMBER/) {
			print $out "PORT_NUMBER \"$port\"\n";
			$done = 1;
		} else {
			print $out $_;
		}
	}

	print $out "\nPORT_NUMBER \"$port\"\n" unless ($done);

}


# The basic algorithm is this:
# hostname => ip addresses
# for each address, ipAddress => hostname2 => ipAddress2
# make sure ipAddress == ipAddress2. If they are all equal, good. If not, bad.
# Here are 2 examples:
# --------------------
# www.yahoo.com => 204.71.202.160, 216.115.105.2, 204.71.200.74,
#                  204.71.200.75, 204.71.200.67, 204.71.200.68
# 204.71.202.160 => w1.yahoo.com => 204.71.202.160
#
# They match. Good.
# --------------------
# yahoo.com => 204.71.200.243, 204.71.200.245
# 204.71.200.243 => UNKNOWN.yahoo.com => does not exist
# 204.71.200.245 => UNKNOWN.yahoo.com => does not exist
# They don't exist. Bad
sub checkDNSLookup {
    use Socket;
    my $host = shift; 
    my ($name,$aliases,$addrtype,$length,@addrs) = gethostbyname($host) or return 0;
    my $ipaddr;
    foreach $ipaddr (@addrs) {
	my $revHost = gethostbyaddr($ipaddr, AF_INET) or return 0;
	my $revHostToIP =  gethostbyname($revHost) or return 0;
	if ($ipaddr != $revHostToIP) {
	    return 0;
	}
    }
    return 1;
}
